package programming.parttwo;

/** Base implementation of Sokoban game 
 *  @author Manuel Pereira 
 *  @author Juan Gomez Romero 
 *  
 *  DO NOT MODIFY THIS CLASS! */

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JFrame;

import programming.parttwo.MySokoban.Cell;

public abstract class Sokoban extends JFrame {
	private static final long serialVersionUID = 1L;
	
	/* Game frame constants */	
	private static final int CELL_SIZE     = 30;
    private static final int BORDER_SIZE   = 50;
    private static final int PADDING_RIGHT = (CELL_SIZE + 1) * 5;
    private int frameHeight;
    private int frameWidth;
    
    /* Board size */
    /** Number of rows of the board*/
    private int rows;
    /** Number of columns of the board */
    private int cols;
    
    /* Game attributes */
    
    /* Colors */
    private static final Color STORAGE_COLOR = Color.GREEN;
    private static final Color PLAYER_COLOR  = Color.BLUE;
    private static final Color EYES_COLOR    = Color.WHITE;
    private static final Color BOX_COLOR     = Color.ORANGE;
    private static final Color EMPTY_COLOR   = Color.BLACK;
    private static final Color WALL_COLOR    = Color.GRAY;

    //--------------------------------------------------------------------
    /** Constructor 
     *  @param rows Number of rows of the board (height) 
     *  @param cols Number of columns of the board (width) */
    public Sokoban(int rows, int cols) {
    	init(rows, cols);
    }
    
    /** Initialization method 
     *  @param rows Number of rows of the board (height) 
     *  @param cols Number of columns of the board (width) */
    private void init(int rows, int cols) {
    	/* Initialize board */
    	this.rows = rows;
    	this.cols = cols;
        
        
        /* Initialize move */
        setMoves(0);
        
        /* Initialize frame */
        frameWidth  = cols * (CELL_SIZE+1) + (2 * BORDER_SIZE) + PADDING_RIGHT;
        frameHeight = rows * (CELL_SIZE+1) + (2 * BORDER_SIZE);
        setSize (frameWidth, frameHeight);
        setTitle("Sokoban");
        setResizable(false);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);        
        
        /* Add key listener */
        final Sokoban s = this;
        addKeyListener(
        		new KeyListener() {  				
					public void keyPressed(KeyEvent e) {
						char key = e.getKeyChar();
        				s.processKey(key);
        				repaint();
					}
					public void keyReleased(KeyEvent e) {}
					public void keyTyped(KeyEvent e) {}																
        		}
    	);    
        		        
        /* Show frame contents */
        this.setVisible(true);
    }

    //--------------------------------------------------------------------
    public void paint(Graphics g) {
        Image buffer = createImage (frameWidth, frameHeight);
        Graphics sg = buffer.getGraphics();
        sg.clearRect(0, 0, frameWidth, frameHeight);
        drawBoard(sg);
        g.drawImage(buffer, 0, 0, this);
    }

    /** Draw board on the frame. 
     *  This method is internally called by paint(Graphics g)
     *  @param g Graphic object to draw on */
    private void drawBoard(Graphics g) {
    	// Border
        g.fillRect(BORDER_SIZE - 2, BORDER_SIZE - 2, (CELL_SIZE+1)* (cols) + 3, (CELL_SIZE+1) * (rows) + 3);

        // Draw cells
        for (int i = 0; i < rows; i++) {
            for (int j = 0; j < cols; j++) {
            	Cell cell = null;
            	try {
            		cell = getCellFromBoard(i, j);
            	} catch(Exception e) {
            		System.out.println("Board has not been initialized yet.");
            		return;
            	}
                switch(cell) {
                    case EMPTY:
                        g.setColor(EMPTY_COLOR);
                        g.fillRect(BORDER_SIZE+(CELL_SIZE+1)*j, BORDER_SIZE+(CELL_SIZE+1)*i, CELL_SIZE, CELL_SIZE);
                        break;
                    case WALL:
                        g.setColor(WALL_COLOR);
                        g.fillRect(BORDER_SIZE+(CELL_SIZE+1)*j, BORDER_SIZE+(CELL_SIZE+1)*i, CELL_SIZE, CELL_SIZE);
                        break;
                    case BOX:
                        g.setColor(BOX_COLOR);
                        g.fillRect(BORDER_SIZE+(CELL_SIZE+1)*j, BORDER_SIZE+(CELL_SIZE+1)*i, CELL_SIZE, CELL_SIZE);
                        break;
                    case STORAGE:
                        g.setColor(STORAGE_COLOR);
                        g.fillArc(BORDER_SIZE+(CELL_SIZE+1)*j, BORDER_SIZE+(CELL_SIZE+1)*i, CELL_SIZE-2, CELL_SIZE-2, 120, 360);
                        break;
                    case STORED_BOX:
                        g.setColor(BOX_COLOR);
                        g.fillRect(BORDER_SIZE+(CELL_SIZE+1)*j, BORDER_SIZE+(CELL_SIZE+1)*i, CELL_SIZE, CELL_SIZE);
                        g.setColor(STORAGE_COLOR);
                        g.drawArc(BORDER_SIZE+(CELL_SIZE+1)*j, BORDER_SIZE+(CELL_SIZE+1)*i, CELL_SIZE-2, CELL_SIZE-2, 120, 360);
                        break;
                    case PLAYER:
                        g.setColor(PLAYER_COLOR);
                        // Body
                        g.fillArc(BORDER_SIZE+(CELL_SIZE+1)*j, BORDER_SIZE+(CELL_SIZE+1)*i, CELL_SIZE-2, CELL_SIZE-2, 0, 180);
                        g.fillRect(BORDER_SIZE+(CELL_SIZE+1)*j, BORDER_SIZE+(CELL_SIZE+1)*i+(CELL_SIZE/2)-1, CELL_SIZE, CELL_SIZE/3);
                        g.fillOval(BORDER_SIZE+(CELL_SIZE+1)*j, BORDER_SIZE+(CELL_SIZE+1)*i+(CELL_SIZE/2)+(CELL_SIZE/6), CELL_SIZE/3, CELL_SIZE/3);
                        g.fillOval(BORDER_SIZE+(CELL_SIZE+1)*j+(CELL_SIZE/3), BORDER_SIZE+(CELL_SIZE+1)*i+(CELL_SIZE/2)+(CELL_SIZE/6), CELL_SIZE/3, CELL_SIZE/3);
                        g.fillOval(BORDER_SIZE+(CELL_SIZE+1)*j+(2*CELL_SIZE/3), BORDER_SIZE+(CELL_SIZE+1)*i+(CELL_SIZE/2)+(CELL_SIZE/6), CELL_SIZE/3, CELL_SIZE/3);
                        // Eyes
                        g.setColor(EYES_COLOR);
                        g.fillOval(BORDER_SIZE+(CELL_SIZE+1)*j+(CELL_SIZE/4), BORDER_SIZE+(CELL_SIZE+1)*i+(CELL_SIZE/4), CELL_SIZE/5, CELL_SIZE/4);
                        g.fillOval(BORDER_SIZE+(CELL_SIZE+1)*j+(2*CELL_SIZE/4), BORDER_SIZE+(CELL_SIZE+1)*i+(CELL_SIZE/4), CELL_SIZE/5, CELL_SIZE/4);
                        g.setColor(EMPTY_COLOR);
                        g.fillOval(BORDER_SIZE+(CELL_SIZE+1)*j+(CELL_SIZE/4)+CELL_SIZE/20, BORDER_SIZE+(CELL_SIZE+1)*i+(CELL_SIZE/4)+CELL_SIZE/8, CELL_SIZE/10, CELL_SIZE/8);
                        g.fillOval(BORDER_SIZE+(CELL_SIZE+1)*j+(2*CELL_SIZE/4)+CELL_SIZE/20, BORDER_SIZE+(CELL_SIZE+1)*i+(CELL_SIZE/4)+CELL_SIZE/8, CELL_SIZE/10, CELL_SIZE/8);
                        break;
                    case PLAYER_AT_STORAGE:
                    	// Storage
                        g.setColor(STORAGE_COLOR);
                        g.fillArc(BORDER_SIZE+(CELL_SIZE+1)*j, BORDER_SIZE+(CELL_SIZE+1)*i, CELL_SIZE-2, CELL_SIZE-2, 120, 360);                    	
                        // Player
                        g.setColor(PLAYER_COLOR);
                        // Body
                        g.fillArc(BORDER_SIZE+(CELL_SIZE+1)*j, BORDER_SIZE+(CELL_SIZE+1)*i, CELL_SIZE-2, CELL_SIZE-2, 0, 180);
                        g.fillRect(BORDER_SIZE+(CELL_SIZE+1)*j, BORDER_SIZE+(CELL_SIZE+1)*i+(CELL_SIZE/2)-1, CELL_SIZE, CELL_SIZE/3);
                        g.fillOval(BORDER_SIZE+(CELL_SIZE+1)*j, BORDER_SIZE+(CELL_SIZE+1)*i+(CELL_SIZE/2)+(CELL_SIZE/6), CELL_SIZE/3, CELL_SIZE/3);
                        g.fillOval(BORDER_SIZE+(CELL_SIZE+1)*j+(CELL_SIZE/3), BORDER_SIZE+(CELL_SIZE+1)*i+(CELL_SIZE/2)+(CELL_SIZE/6), CELL_SIZE/3, CELL_SIZE/3);
                        g.fillOval(BORDER_SIZE+(CELL_SIZE+1)*j+(2*CELL_SIZE/3), BORDER_SIZE+(CELL_SIZE+1)*i+(CELL_SIZE/2)+(CELL_SIZE/6), CELL_SIZE/3, CELL_SIZE/3);
                        // Eyes
                        g.setColor(STORAGE_COLOR);
                        g.fillOval(BORDER_SIZE+(CELL_SIZE+1)*j+(CELL_SIZE/4), BORDER_SIZE+(CELL_SIZE+1)*i+(CELL_SIZE/4), CELL_SIZE/5, CELL_SIZE/4);
                        g.fillOval(BORDER_SIZE+(CELL_SIZE+1)*j+(2*CELL_SIZE/4), BORDER_SIZE+(CELL_SIZE+1)*i+(CELL_SIZE/4), CELL_SIZE/5, CELL_SIZE/4);
                        g.setColor(EMPTY_COLOR);
                        g.fillOval(BORDER_SIZE+(CELL_SIZE+1)*j+(CELL_SIZE/4)+CELL_SIZE/20, BORDER_SIZE+(CELL_SIZE+1)*i+(CELL_SIZE/4)+CELL_SIZE/8, CELL_SIZE/10, CELL_SIZE/8);
                        g.fillOval(BORDER_SIZE+(CELL_SIZE+1)*j+(2*CELL_SIZE/4)+CELL_SIZE/20, BORDER_SIZE+(CELL_SIZE+1)*i+(CELL_SIZE/4)+CELL_SIZE/8, CELL_SIZE/10, CELL_SIZE/8);
                        break;
                }
            }
        }

        // Moves text
        g.setColor(Color.black);
        g.fillRect(BORDER_SIZE + (CELL_SIZE+1)* (cols +1) - 5, BORDER_SIZE+ (CELL_SIZE+1)* (rows - 3) + 7, PADDING_RIGHT - 2, CELL_SIZE);
        g.setColor(Color.white);
        g.drawString("Moves: " + getMoves(), BORDER_SIZE+ (CELL_SIZE+1)* (cols +1) , BORDER_SIZE+ (CELL_SIZE+1)* (rows - 2));
    }
    
    //--------------------------------------------------------------------    
   

    //------------------------------------------------
    public abstract void processKey(char tecla);
    public abstract Cell getCellFromBoard(int row, int col);
    public abstract void setMoves(int moves);
    public abstract int getMoves();
   
}


